<?php

namespace bhr\Frontend\Plugins\Gf;

if(!defined('ABSPATH')) exit;

use bhr\Frontend\Model\AbstractContactModel;
use SALESmanago\Entity\Contact\Contact;
use SALESmanago\Exception\Exception;

class GfContactModel extends AbstractContactModel
{
    const
        CONF                 = 'confirmation',
        CONF_DOUBLE          = 'double',
        PROPS                = 'properties',
        PREFIX_FIELD         = 'sm-',
        CONSENT_FIELD        = 'sm-consent-',

        FIELDS               = 'fields',
        NAME_FIELD           = 'name',
        LAST_NAME_FIELD      = 'lastname',
        EMAIL_FIELD          = 'email',
        PHONE_FIELD          = 'phone',
        COMPANY_FIELD        = 'company',
        ADDRESS_FIELD        = 'address',
        ZIPCODE_FIELD        = 'zipcode',
        CITY_FIELD           = 'city',
        COUNTRY_FIELD        = 'country',
        OPT_IN_FIELD         = 'optin',

        FORCE_OPTIN          = 'forceOptIn',

        CONTACT_CONSENTS     = 'consents',

        PROVINCE_FIELD       = 'province';

    protected $currentFormConfig;

    public function __construct($PlatformSettings)
    {
        //do not continue without settings
        if(empty($PlatformSettings) || empty($PlatformSettings->PluginGf)) {
            return false;
        }
        //create an Abstract Contact
        parent::__construct($PlatformSettings, $PlatformSettings->PluginGf);
        return true;
    }

    /**
     * @param $form
     * @return bool
     */
    public function setCurrentFormConfig($form)
    {
        //Get config (tags, owner) for submitted form
        $id = intval($form['id']);
        if(isset($this->PluginSettings->forms->$id)) {
            $this->currentFormConfig = $this->PluginSettings->forms->$id;
            return true;
        }
        return false;
    }

    /**
     * @param $formValues
     * @param $formSetup
     * @return Contact|null
     * @throws Exception
     */
    public function parseContact($formValues, $formSetup)
    {
        try {
            //Translate Form Values and Form Setup into a single associative array
            $formFieldsValues = $this->translateFieldsAndValues($formValues, $formSetup);

            //No email no fun
            if (!isset($formFieldsValues['email'])
                || empty($formFieldsValues['email'])) {
                return null;
            }

            /* Contact */
            $this->Contact
                ->setEmail($formFieldsValues['email'])
                ->setName(isset($formFieldsValues['name'])
                        ? trim(implode(' ', $formFieldsValues['name']))
                        : '')
                ->setPhone(isset($formFieldsValues[self::PHONE_FIELD])
                    ? $formFieldsValues[self::PHONE_FIELD]
                    : '')
                ->setCompany(isset($formFieldsValues[self::COMPANY_FIELD])
                    ? $formFieldsValues[self::COMPANY_FIELD]
                    : '');

            /* Address */
            $this->Address
                ->setStreetAddress(isset($formFieldsValues[self::ADDRESS_FIELD])
                    ? trim(implode(' ', $formFieldsValues[self::ADDRESS_FIELD]))
                    : '')
                ->setCountry(isset($formFieldsValues[self::COUNTRY_FIELD])
                    ? $formFieldsValues[self::COUNTRY_FIELD]
                    : '')
                ->setCity(isset($formFieldsValues[self::CITY_FIELD])
                    ? $formFieldsValues[self::CITY_FIELD]
                    : '')
                ->setZipCode(isset($formFieldsValues[self::ZIPCODE_FIELD])
                    ? $formFieldsValues[self::ZIPCODE_FIELD]
                    : '')
                ->setProvince(isset($formFieldsValues[self::PROVINCE_FIELD])
                    ? $formFieldsValues[self::PROVINCE_FIELD]
                    : '');

            /* Options */
            $this->setLanguage();
            $this->Options
                ->setTags(isset($this->currentFormConfig->tags)
                        ? $this->currentFormConfig->tags
                        : '')
                ->setRemoveTags(isset($this->currentFormConfig->tagsToRemove)
                        ? $this->currentFormConfig->tagsToRemove
                        : '');
            if(!empty($formFieldsValues['opt-in']) && boolval($formFieldsValues['opt-in'])) {
                $this->Options->setIsSubscribes(true);
                $this->Options->setIsSubscriptionStatusNoChange(false);
            }

            /* Properties */
            $this->Properties
                ->setItems(isset($formFieldsValues['properties'])
                    ? $formFieldsValues['properties']
                    : array());

            return $this->Contact;
        } catch (\Exception $e) {
            throw new Exception($e->getMessage());
        }
    }

    /**
     * @param $formValues
     * @param $formSetup
     * @return array
     */
    private function translateFieldsAndValues($formValues, $formSetup)
    {
        $output = array();
        foreach($formSetup['fields'] as $field) {
            //If field has no name or id
            if(empty($field->adminLabel) || empty($field->id)) {
                continue;
            }

            //Get field's value
            $value = rgar($formValues, (string)$field->id);

            //Check if field is a custom property
            if(!empty($this->PluginSettings->properties)
            && in_array($field->adminLabel, $this->PluginSettings->properties)) {
                //get array of inputs. inputs are arrays (id, label, name)
                $inputs = $field->get_entry_inputs();

                if (is_array($inputs)) {

                    //tmp array to hold values of input array. Workaround for array_column incompatibility
                    $inputValues = [];

                    foreach ($inputs as $input) {

                        //magic happens and we get the label
                        $value = rgar($formValues, (string)$input['id']);
                        if (!empty($value)) {
                            $inputValues[] = $value;
                        }
                    }
                    $output['properties'][$field->adminLabel]
                        = mb_substr(implode(',', $inputValues), 0, self::MAX_STRING_LENGTH);
                } else {
                    $output['properties'][$field->adminLabel] = mb_substr($value, 0, self::MAX_STRING_LENGTH);
                }
                continue;
            }

            //Basic fields should start with 'sm-'
            if(strpos($field->adminLabel, self::PREFIX_FIELD) === false
            && strpos($field->adminLabel, 'sm_') === false) {
                continue;
            }

            //Remove 'sm-' from field's name
            $key = strtolower(substr($field->adminLabel, 3));

            //Correct for common mistakes
            switch($key) {
                case 'name':
                case 'firstname':
                case 'firstName':
                case 'lastName':
                case 'last_name':
                case 'lastname':
                case 'first_name':
                    $output['name'][] = $value;
                    break;
                case 'e-mail':
                    $output['email'] = $value;
                    break;
                case 'address':
                case 'streetaddress':
                case 'streetAddress':
                case 'street-address':
                case 'address1':
                case 'address-1':
                case 'address_1':
                case 'streetaddress2':
                case 'streetAddress2':
                case 'street-address-2':
                case 'address2':
                case 'address-2':
                case 'address_2':
                    $output['address'][] = $value;
                    break;
                case 'optin':
                case 'opt-in':
                    $output['opt-in'] = empty($value)
                        ? rgar($formValues, (string)$field->get_entry_inputs()[0]['id'])
                        : $value;
                    break;

                //Set field's value to output array
                default:
                    $output[$key] =  is_array($value) ? implode(', ', $value) : $value;
                    break;
            }
        }
        return $output;
    }
}