<?php

namespace bhr\Frontend\Plugins\Wc;

if(!defined('ABSPATH')) exit;

use bhr\Frontend\Plugins\Wc\WcContactModel as ContactModel;
use bhr\Frontend\Plugins\Wc\WcEventModel as EventModel;
use SALESmanago\Exception\Exception;

use bhr\Frontend\Controller\TransferController;

class WcController
{
    const PREVENT_EVENT_DUPLICATION_TIME = 30;

    private $TransferController;
    private $ContactModel;
    private $EventModel;

    private $preventEventDuplication = false;

    public function __construct($PlatformSettings, TransferController $TransferController)
    {
        $this->TransferController = $TransferController;
        if (!$this->ContactModel = new ContactModel($PlatformSettings)) {
            return false;
        }
        $this->EventModel = new EventModel();

        if (isset($PlatformSettings->PluginWc->preventEventDuplication)) {
            $this->preventEventDuplication = boolval($PlatformSettings->PluginWc->preventEventDuplication);
        }

        return $this;
    }

    /**
     * @param $userId
     * @return bool
     */
    public function createUser($userId)
    {
        try {
            //Populate new Contact Model with fields from submitted data
            if ($this->ContactModel->parseCustomerFromPost()) {
                return $this->TransferController->transferContact($this->ContactModel->get());
            } elseif ($this->ContactModel->parseContact($userId, ContactModel::ID)) {
                return $this->TransferController->transferContact($this->ContactModel->get());
            }
            return false;

        } catch (\Exception $e) {
            error_log(print_r($e->getMessage(), true));
        }
        return false;
    }

    /**
     * @param $userLogin
     * @return bool
     */
    public function loginUser($userLogin)
    {
        try {
            $user = (!get_user_by('email', $userLogin))
                ? get_user_by('login', $userLogin)
                : get_user_by('email', $userLogin);

            if ($user && empty($user->get_role_caps()['customer'])) {
                return false;
            }

            if ($this->ContactModel->parseContact($userLogin, ContactModel::LOGIN)) {
                $this->ContactModel->setTagsFromConfig(ContactModel::TAGS_LOGIN);
                return $this->TransferController->transferContact($this->ContactModel->get());
            }

            return false;
        } catch (\Exception $e) {
            error_log(print_r($e->getMessage(), true));
        }
        return false;
    }

    /**
     * Parse woocommerce user to SM contact, send contact to SM
     * Set smclient for contact
     * @param $userId - registered user id
     * @return bool
     */
    public function registerUser($userId)
    {
        try {
            if ($this->ContactModel->parseContact($userId, ContactModel::ID)) {
                $this->ContactModel->setTagsFromConfig(ContactModel::TAGS_REGISTRATION);
                return $this->TransferController->transferContact($this->ContactModel->get());
            }
            return false;
        } catch (\Exception $e) {
            error_log(print_r($e->getMessage(), true));
        }
        return false;
    }

    /**
     * Parse & send order data as SM external event Purchase
     *
     * @param mixed $order  WC order or wc_order_id
     * @return bool         When no orders exist or get order with no processed statuses
     * @log Exception       log to wp logs an \Exception
     */
    public function purchase($order)
    {
        try {
            if (empty($order)) {
                return true;
            }

            try {
                if ($this->preventEventDuplication) {
                    session_start();
                    if (isset($_SESSION['salesmanagoLastEvent'])
                        && $_SESSION['salesmanagoLastEvent'] + self::PREVENT_EVENT_DUPLICATION_TIME > time()) {
                        return false;
                    } else {
                        $_SESSION['salesmanagoLastEvent'] = time();
                    }
                }
            } catch (\Exception $e) {
                /* Silence is golden */
            }

            $order = is_int($order) ? wc_get_order($order) : $order;

            // Populate Contact Model with fields from submitted data
            if (!$this->ContactModel->parseCustomerFromPost()) {
                if (!$this->ContactModel->parseContact($order->get_id())) {
                    return false;
                }
            }

            $order->get_user()
                ? $this->ContactModel->setTagsFromConfig(ContactModel::TAGS_PURCHASE)
                : $this->ContactModel->setTagsFromConfig(ContactModel::TAGS_GUEST_PURCHASE);

            $client[ContactModel::EMAIL]     = $this->ContactModel->getClientEmail();
            $client[ContactModel::SM_CLIENT] = ContactModel::getSmClient();
            if (empty($client[ContactModel::SM_CLIENT]) && empty($client[ContactModel::EMAIL])) {
                return false;
            }

            $products = $this->EventModel->getProductsFromOrder($order);
            if (!$products) {
                return false;
            }

            $this->EventModel->bindEvent(
                $client,
                EventModel::EVENT_TYPE_PURCHASE,
                $products,
                ''
            );

            return $this->TransferController->transferBoth($this->ContactModel->get(), $this->EventModel->get());
        } catch (\Exception $e) {
            error_log(print_r($e->getMessage(), true));
        }
        return false;
    }

    /**
     * @return bool
     */
    public function addToCart()
    {
        try {
            $client[ContactModel::SM_CLIENT] = ContactModel::getSmClient();
            $client[ContactModel::EMAIL]     = $this->ContactModel->getClientEmail();
            if (empty($client[ContactModel::SM_CLIENT]) && empty($client[ContactModel::EMAIL])) {
                return false;
            }

            $products = $this->EventModel->getProductsFromCart();
            if (!$products) {
                return false;
            }

            $smEvent = $this->EventModel->getSmEvent();

            $this->EventModel->bindEvent(
                $client,
                EventModel::EVENT_TYPE_CART,
                $products,
                $smEvent
            );
            return $this->TransferController->transferEvent($this->EventModel->get());

        } catch (Exception $e) {
            error_log(print_r($e->getMessage(), true));
        } catch (\Exception $e) {
            error_log(print_r($e->getMessage(), true));
        }
        return false;
    }

    /**
     * @param $orderId
     * @return bool
     */
    public function orderStatusChanged($orderId)
    {
        try {
            $eventHook = current_action();
            switch($eventHook) {
                case 'woocommerce_order_status_cancelled':
                    $eventType = EventModel::EVENT_TYPE_CANCELLATION;
                    break;
                case 'woocommerce_order_status_refunded':
                    $eventType = EventModel::EVENT_TYPE_RETURN;
                    break;
                default:
                    $eventType = EventModel::EVENT_TYPE_DEFAULT;
            }

            if (!$this->ContactModel->parseCustomer($orderId)) {
                return false;
            }

            $client["email"] = $this->ContactModel->get()->getEmail();
            $client["smclient"] = ContactModel::getSmClient();
            if (empty($client["email"]) && empty($client["smclient"])) {
                return false;
            }

            $smEvent = $this->EventModel->getSmEvent();


            $products = $this->EventModel->getProductsFromOrder($orderId);
            if (!$products) {
                return false;
            }

            $this->EventModel->bindEvent(
                $client,
                $eventType,
                $products,
                $smEvent
            );
            return $this->TransferController->transferEvent($this->EventModel->get());

        } catch (Exception $e) {
            error_log(print_r($e->getMessage(), true));
        } catch (\Exception $e) {
            error_log(print_r($e->getMessage(), true));
        }
        return false;
    }
}
